@echo off
Rem ---------------------------------------------------------------------------
Rem JOnAS: Java(TM) Open Application Server
Rem Copyright (C) 2006 Bull S.A.S.
Rem Contact: jonas-team@objectweb.org
Rem
Rem This library is free software; you can redistribute it and/or
Rem modify it under the terms of the GNU Lesser General Public
Rem License as published by the Free Software Foundation; either
Rem version 2.1 of the License, or any later version.
Rem
Rem This library is distributed in the hope that it will be useful,
Rem but WITHOUT ANY WARRANTY; without even the implied warranty of
Rem MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Rem Lesser General Public License for more details.
Rem
Rem You should have received a copy of the GNU Lesser General Public
Rem License along with this library; if not, write to the Free Software
Rem Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
Rem USA
Rem
Rem Initial developer(s): Jerome Pioux - Benoit Pelletier
Rem ---------------------------------------------------------------------------
Rem $Id: $
Rem ---------------------------------------------------------------------------

:: Start, Stop JONAS cluster nodes
:: Usage: cl [-n <node>] start | stop

:: If set to yes, each jonas instance is started in a different window
set SEPARATE_WINDOWS=yes

:: number of nodes to start (if not exists)
if "%JCL_NUMBER_OF_NODES%"=="" (
    set /a NUMBER_OF_NODES=ZZ1
) else (
    set /a NUMBER_OF_NODES=%JCL_NUMBER_OF_NODES%
)

:: base of the JONAS_BASE directories (if not exists)
if "%JCL_BASE_PREFIX%"=="" (
    set /a BASE_PREFIX=ZZ2
) else (
    set /a BASE_PREFIX=%JCL_BASE_PREFIX%
)

:: discovery.domainName (if not exists)
if "%JCL_DOMAIN_NAME%"=="" (
    set /a DOMAIN_NAME=ZZ3
) else (
    set /a DOMAIN_NAME=%JCL_DOMAIN_NAME%
)

:: Full pathname where the EAR to deploy is (put NULL if none) (if not exists)
if "%JCL_EAR_FILE%"=="" (
    set /a EAR_FILE=ZZ4
) else (
    set /a EAR_FILE=%JCL_EAR_FILE%
)

:: base of the nodes name (if not exists)
if "%JCL_NODE_NAME_PREFIX%"=="" (
    set /a NODE_NAME_PREFIX=ZZ5
) else (
    set /a NODE_NAME_PREFIX=%JCL_NODE_NAME_PREFIX%
)

:: cluster daemon dir (if not exists)
if "%JCL_CLUSTER_DAEMON_DIR%"=="" (
    set /a CLUSTER_DAEMON_DIR=ZZ6
) else (
    set /a CLUSTER_DAEMON_DIR=%JCL_CLUSTER_DAEMON_DIR%
)

:: db dir(if not exists)
if "%JCL_DB_DIR%"=="" (
    set /a DB_DIR=ZZ7
) else (
    set /a DB_DIR=%JCL_DB_DIR%

:: db name(if not exists)
if "%JCL_DB_NAME%"=="" (
    set /a DB_NAME=ZZ8
) else (
    set /a DB_NAME=%JCL_DB_NAME%
)

:: master dir(if not exists)
if "%JCL_MASTER_DIR%"=="" (
    set /a MASTER_DIR=ZZ9
) else (
    set /a MASTER_DIR=%JCL_MASTER_DIR%

:: master name(if not exists)
if "%JCL_MASTER_NAME%"=="" (
    set /a MASTER_NAME=ZZ8
) else (
    set /a MASTER_NAME=%JCL_MASTER_NAME%
)

:: ------------------------------------------
:: NO NEED TO MODIFY ANYTHING BELOW THIS LINE
:: ------------------------------------------
setlocal ENABLEDELAYEDEXPANSION

:: CHECK if JONAS_ROOT is set
if "%JONAS_ROOT%X"=="X" (
    echo Error: JONAS_ROOT is not set
    goto endScript
)

:: Set default values for sampleCluster2
if "%NUMBER_OF_NODES%"=="ZZ1" (
    set /a MAX_NODES=4
) else (
    set /a MAX_NODES=%NUMBER_OF_NODES%
)

if "%BASE_PREFIX%"=="ZZ2" set BASE_PREFIX=wk
if "%DOMAIN_NAME%"=="ZZ3" set DOMAIN_NAME=sampleCluster2Domain
if "%EAR_FILE%"=="ZZ4"    set EAR_FILE=%JONAS_ROOT%\examples\sampleCluster2\output\apps\sampleCluster2.ear
if "%NODE_NAME_PREFIX%"=="ZZ5" set NODE_NAME_PREFIX=node
if "%CLUSTER_DAEMON_DIR%"=="ZZ6" set CLUSTER_DAEMON_DIR=cd
if "%DB_DIR%"=="ZZ7" set DB_DIR=db
if "%DB_NAME%"=="ZZ8" set DB_NAME=db
if "%MASTER_DIR%"=="ZZ9" set MASTER_DIR=db
if "%MASTER_NAME%"=="ZZ10" set MASTER_NAME=db

set /a node=0

:: Parse command line
:: ------------------
:: Help the "dos cmdline way"
if "%1X"=="/?X" goto usage

set PARAM1=%1
Set SUB_PARAM1=%PARAM1:~0,2%

rem -n is used to address a specific node
if /I "%SUB_PARAM1%"=="-n" (
    rem Unix style, no space between -n and its value
    if not "%PARAM1%"=="%SUB_PARAM1%" (
        set node=%PARAM1:~2,4%
        set CMD=%2
        set OPT=%3
    rem space between -n and its value
    ) else (
        set node=%2
        set CMD=%3
        set OPT=%4
    )
rem command applies to entire cluster
) else (
    set CMD=%1
    set OPT=%2
)

:: Check syntax - if incorrect show usage
if not "%OPT%X"=="X" goto usage

:: If supported command, goto action otherwise show usage
if /I "%CMD%"=="start"  goto action
if /I "%CMD%"=="stop"   goto action
goto usage

:action
:: Check the node validity
if %node% GTR %MAX_NODES% (
	echo Error: -n ^<node^> must be between 1 and %MAX_NODES%
    goto endScript
)

:: Set JONAS_BASE NAMES
for /l %%a in (1,1,%MAX_NODES%) do (
    set jonas_base[%%a]=%BASE_PREFIX%%%a
)

:: Go do the work
call :%CMD% %node%

rem ------------------------------ END SCRIPT -----------------------------
:endScript
echo.
:end
exit /b 0


:: ****************************** SUBROUTINES *****************************

:: -----
:: START
:: -----
:start

set WIN=
if "%SEPARATE_WINDOWS%"=="yes" set WIN=-win

:: start cluster daemon
set  /a JONAS_BASE=!CLUSTER_DAEMON_DIR!
call jclusterd

:: start MASTER INSTANCE
set  /a JONAS_BASE=!MASTER_DIR!
call jonas start !WIN! -n !MASTER_NAME! -Ddomain.name=%DOMAIN_NAME%

:: start DB INSTANCE
set  /a JONAS_BASE=!DB_DIR!
call jonas start !WIN! -n !DB_NAME! -Ddomain.name=%DOMAIN_NAME%


set /a Start=%node%
set /a End=%node%

if %node% EQU 0 (
    set /a Start=1
    set /a End=%MAX_NODES%
)

for /l %%a in (%Start%,1,%End%) do (
    set JONAS_BASE=!jonas_base[%%a]!
    set CATALINA_BASE=!JONAS_BASE!

    echo.
    echo *** Starting JONAS %NODE_NAME_PREFIX%%%a

    rem Save and clean up logs
	rmdir /S /Q !JONAS_BASE!\logs\old\* 2>NUL
	mkdir !JONAS_BASE!\logs\old 2>NUL
	xcopy /Q /Y !JONAS_BASE!\logs\* !JONAS_BASE!\logs\old >NUL

    rem Eventually deploy
	if not "%EAR_FILE%"=="NULL" (
        if exist %EAR_FILE% (
            copy /Y %EAR_FILE% !JONAS_BASE!\apps\autoload >NUL
        )
    )

    rem Start it
    call jonas start !WIN! -n %NODE_NAME_PREFIX%%%a -Ddomain.name=%DOMAIN_NAME%
)
goto endScript

:: ----
:: STOP
:: ----
:stop

:: stop cluster daemon
set  /a JONAS_BASE=!CLUSTER_DAEMON_DIR!
call jclusterd stop

set /a Start=%node%
set /a End=%node%

if %node% EQU 0 (
    set /a Start=1
    set /a End=%MAX_NODES%
)

for /l %%a in (%Start%,1,%End%) do (
    set JONAS_BASE=!jonas_base[%%a]!
    set CATALINA_BASE=!JONAS_BASE!

    echo.
	echo *** Stopping JONAS %NODE_NAME_PREFIX%%%a for JONAS_BASE=!JONAS_BASE!
	jonas stop -n %NODE_NAME_PREFIX%%%a
)

:: stop DB INSTANCE
set  /a JONAS_BASE=!DB_DIR!
call jonas stop -n !DB_NAME!

:: stop MASTER INSTANCE
set  /a JONAS_BASE=!MASTER_DIR!
call jonas stop -n !MASTER_NAME!

goto endScript

:: -----
:: USAGE
:: -----
:usage
echo usage: cl ^[-n^<node^>^] start ^| stop
goto endScript
