#!/bin/bash
# ---------------------------------------------------------------------------
# JOnAS: Java(TM) Open Application Server
# Copyright (C) 2006 Bull S.A.S.
# Contact: jonas-team@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Jerome Pioux - Benoit Pelletier
# ---------------------------------------------------------------------------
# $Id: $
# ---------------------------------------------------------------------------

# Status|Start|Stop|Kill JONAS cluster node(s)
# Usage: cl [-n<node>] [status] | < [-f] kill | [-d] start | stop>"

# If XWINDOWS=yes or YES exists in the user env., then use -win on startup
#XWINDOWS=${XWINDOWS:-no}
XWINDOWS=yes

# number of nodes to start (if not exists)
[[ "$JCL_NUMBER_OF_NODES" = "" ]] && JCL_NUMBER_OF_NODES=ZZ1

# base of the JONAS_BASE directories (if not exists)
[[ "$JCL_BASE_PREFIX" = "" ]] && JCL_BASE_PREFIX=ZZ2

# discovery.domainName (if not exists)
[[ "$JCL_DOMAIN_NAME" = "" ]] && JCL_DOMAIN_NAME=ZZ3

# Full pathname where the EAR to deploy is (put NULL if none) (if not exists)
[[ "$JCL_EAR_FILE" = "" ]] && JCL_EAR_FILE=ZZ4

# base of the nodes name (if not exists)
[[ "$JCL_NODE_NAME_PREFIX" = "" ]] && JCL_NODE_NAME_PREFIX=ZZ5

# base of the cluster daemon (if not exists)
[[ "$JCL_CLUSTER_DAEMON_DIR" = "" ]] && JCL_CLUSTER_DAEMON_DIR=ZZ6

# base of the db instance (if not exists)
[[ "$JCL_DB_DIR" = "" ]] && JCL_DB_DIR=ZZ7

# name of the db instance (if not exists)
[[ "$JCL_DB_NAME" = "" ]] && JCL_DB_NAME=ZZ8

# base of the master instance (if not exists)
[[ "$JCL_MASTER_DIR" = "" ]] && JCL_MASTER_DIR=ZZ9

# name of the master instance (if not exists)
[[ "$JCL_MASTER_NAME" = "" ]] && JCL_MASTER_NAME=ZZ10

# base port for debug (if not exists)
[[ "$JCL_BASE_DEBUG_PORT" = "" ]] && JCL_BASE_DEBUG_PORT=ZZ11

# ------------------------------------------
# NO NEED TO MODIFY ANYTHING BELOW THIS LINE
# ------------------------------------------

# Default values
[[ "$JCL_NUMBER_OF_NODES" = "ZZ1" ]] && JCL_NUMBER_OF_NODES=4
[[ "$JCL_BASE_PREFIX" = "ZZ2" ]] && JCL_BASE_PREFIX=wk
[[ "$JCL_DOMAIN_NAME" = "ZZ3" ]] && JCL_DOMAIN_NAME=sampleCluster2Domain
[[ "$JCL_EAR_FILE" = "ZZ4" ]] && JCL_EAR_FILE=$JONAS_ROOT/examples/sampleCluster2/output/apps/sampleCluster2.ear
[[ "$JCL_NODE_NAME_PREFIX" = "ZZ5" ]] && JCL_NODE_NAME_PREFIX=node
[[ "$JCL_CLUSTER_DAEMON_DIR" = "ZZ6" ]] && JCL_CLUSTER_DAEMON_DIR=cd
[[ "$JCL_DB_DIR" = "ZZ7" ]] && JCL_DB_DIR=db
[[ "$JCL_DB_NAME" = "ZZ8" ]] && JCL_DB_NAME=db
[[ "$JCL_MASTER_DIR" = "ZZ9" ]] && JCL_MASTER_DIR=master
[[ "$JCL_MASTER_NAME" = "ZZ10" ]] && JCL_MASTER_NAME=master
[[ "$JCL_BASE_DEBUG_PORT" = "ZZ11" ]] && JCL_BASE_DEBUG_PORT=414

# -----
# USAGE
# -----
usage()
{
	echo -e "usage: cl [-n<node>] [status] | < [-f] kill | [-d] start | stop>\n"
	exit 1
}

# ---------------------
# START cluster daemon
# ---------------------
startCd()
{

    export JONAS_BASE=$CLUSTER_HOME/${JCL_CLUSTER_DAEMON_DIR}
    echo -e "\n\n*** Starting cluster daemon"
	xterm -title $HOSTNAME:clusterd -geometry 110x16 -sb -e jclusterd start &
	return
}

# ---------------------
# START the DB instance
# ---------------------
startDb()
{
    export JONAS_BASE=$CLUSTER_HOME/${JCL_DB_DIR}
    export CATALINA_BASE=$JONAS_BASE
    echo -e "\n\n*** Starting db instance"
	echo -e "(jonas start -n $JCL_DB_NAME -Ddomain.name=$JCL_DOMAIN_NAME)\n"
	WIN=""
    if [[ "$XWINDOWS" = "yes" || "$XWINDOWS" = "YES" ]]; then
	   WIN="-win"
	fi
	jonas start $WIN -n $JCL_DB_NAME -Ddomain.name=$JCL_DOMAIN_NAME
}

# ---------------------
# START the Master instance
# ---------------------
startMaster()
{
    export JONAS_BASE=$CLUSTER_HOME/${JCL_MASTER_DIR}
    export CATALINA_BASE=$JONAS_BASE
    echo -e "\n\n*** Starting master instance"
	echo -e "(jonas start -n $JCL_MASTER_NAME -Ddomain.name=$JCL_DOMAIN_NAME)\n"
	WIN=""
    if [[ "$XWINDOWS" = "yes" || "$XWINDOWS" = "YES" ]]; then
	   WIN="-win"
	fi
	jonas start $WIN -n $JCL_MASTER_NAME -Ddomain.name=$JCL_DOMAIN_NAME
}

# ---------------------
# STOP cluster daemon
# ---------------------
stopCd()
{
    export JONAS_BASE=$CLUSTER_HOME/${JCL_CLUSTER_DAEMON_DIR}
    echo -e "\n\n*** Stopping cluster daemon"
	jclusterd stop
	return
}

# ---------------------
# STOP the DB instance
# ---------------------
stopDb()
{
    export JONAS_BASE=$CLUSTER_HOME/${JCL_DB_DIR}
    export CATALINA_BASE=$JONAS_BASE
    echo -e "\n\n*** Stopping db instance"
	echo -e "(jonas stop -n $JCL_DB_NAME)\n"
	jonas stop -n $JCL_DB_NAME
}

# ---------------------
# STOP the MASTER instance
# ---------------------
stopMaster()
{
    export JONAS_BASE=$CLUSTER_HOME/${JCL_MASTER_DIR}
    export CATALINA_BASE=$JONAS_BASE
    echo -e "\n\n*** Stopping master instance"
	echo -e "(jonas stop -n $JCL_MASTER_NAME)\n"
	jonas stop -n $JCL_MASTER_NAME
}


# -----
# START
# -----
start()
{
	typeset -i i=$1 N=$1

	if [[ "$i" = "0" ]]; then
		i=1 && N=$JCL_NUMBER_OF_NODES
		startCd
		startMaster
		startDb
	fi

    while [[ $i -le $N ]]; do
		export JONAS_BASE=$CLUSTER_HOME/${jonas_base[${i}]}
		export CATALINA_BASE=$JONAS_BASE
		echo -e "\n\n*** Starting JONAS node$i"

		# Save and clean up logs
		rm -rf $JONAS_BASE/logs/old/*
		mkdir -p $JONAS_BASE/logs/old
		mv $JONAS_BASE/logs/* $JONAS_BASE/logs/old 2>/dev/null

		# Eventually deploy
		if [[ "$JCL_EAR_FILE" != "NULL" && -f $JCL_EAR_FILE ]]; then
			cp $JCL_EAR_FILE $JONAS_BASE/apps/autoload
		fi

		# Eventually deploy HA 
		if [[ "$JCL_HA_EAR_FILE" != "NULL" && -f $JCL_HA_EAR_FILE ]]; then
			cp $JCL_HA_EAR_FILE $JONAS_BASE/apps/autoload
		fi

		WIN=""
		if [[ "$XWINDOWS" = "yes" || "$XWINDOWS" = "YES" ]]; then
			WIN="-win"
		fi

		DEBUG=""
		if [ "$debug" = "true" ]; then
			DEBUG="-debug -p $JCL_BASE_DEBUG_PORT$i"
		fi

		echo -e "(jonas start -n $JCL_NODE_NAME_PREFIX$i -Ddomain.name=$JCL_DOMAIN_NAME)\n"
		jonas start $WIN $DEBUG -n $JCL_NODE_NAME_PREFIX$i -Ddomain.name=$JCL_DOMAIN_NAME
		i=i+1
	done
	return
}


# ----
# STOP
# ----
stop()
{
	typeset -i i=$1 N=$1

	if [[ "$i" = "0" ]]; then
		i=1 && N=$JCL_NUMBER_OF_NODES
		stopCd
		stopDb
		stopMaster
	fi

	while [[ $i -le $N ]]; do
		export JONAS_BASE=$CLUSTER_HOME/${jonas_base[${i}]}
	    export CATALINA_BASE=$JONAS_BASE
		echo -e "\n*** Stopping JONAS $JCL_NODE_NAME_PREFIX$i for JONAS_BASE=$JONAS_BASE"
		jonas stop -n $JCL_NODE_NAME_PREFIX$i
		i=i+1
	done
	return
}

# ----
# KILL
# ----
kill()
{
	typeset -i i=$1 N=$1

	if [[ "$i" = "0" ]]; then
		i=1 && N=$JCL_NUMBER_OF_NODES
	fi

    while [[ $i -le $N ]]; do
        export JONAS_BASE=$CLUSTER_HOME/${jonas_base[${i}]}
   		export CATALINA_BASE=$JONAS_BASE
		proc=$(ps -eo "%p|%a" | grep Djonas.name=$JCL_NODE_NAME_PREFIX$i | grep -v grep | sed -e 's/  *//g')
		pid=${proc%%|*}
		if [[ ! -z $pid ]]; then
        	echo -e "\n*** Killing JONAS $JCL_NODE_NAME_PREFIX$i for JONAS_BASE=$JONAS_BASE"
			echo -e "(kill -$force $pid)"
			/bin/kill -$force $pid
		fi
        i=i+1
    done
    return
}


# ------
# STATUS
# ------
status()
{
	typeset -i i=$1 N=$1

	if [[ "$i" = "0" ]]; then
		i=1 && N=$JCL_NUMBER_OF_NODES
	fi

	while [[ $i -le $N ]]; do
		echo -e "\tJONAS $JCL_NODE_NAME_PREFIX$i is \c"
		proc=$(ps -eo "%p|%a" | grep Djonas.name=$JCL_NODE_NAME_PREFIX$i | grep -v grep | sed -e 's/  *//g')
		pid=${proc%%|*}
		if [[ ! -z $pid ]]; then
        	echo -e "running [$pid]"
		else
        	echo -e "not running"
		fi
		#ps -ef | grep "Djonas.name=$JCL_NODE_NAME_PREFIX$i" | grep -v grep >/dev/null 2>&1
		#[[ $? -eq 0 ]] && echo "running" || echo "not running"
		i=i+1
	done
	return
}

# -----
# Main
# -----
typeset -i n
node=0
force=15
debug=false
echo

if [[ ! -n $JONAS_ROOT ]]; then
	echo -e "Error: JONAS_ROOT not defined\n"
	exit 2
fi

# Read inputs
if [[ $# -ne 0 ]]; then
	while getopts fn:d c
	do
		case $c in
			# Nodes
			n)
				typeset -i node=$OPTARG
				;;
			f)
				force=9
				;;
			d)
				debug=true
				;;
			*)
				usage
				;;
		esac
    done
    shift `expr $OPTIND - 1`
    action="$@"
fi

# Check node
if [[ $node -lt 0 || $node -gt $JCL_NUMBER_OF_NODES ]]; then
	echo -e "Error: -n<node> must be between 1 and $JCL_NUMBER_OF_NODES\n"
	exit 3
fi

# Set BASE NAMES
typeset -i i=1
while [[ $i -le $JCL_NUMBER_OF_NODES ]]; do
	jonas_base[${i}]=${JCL_BASE_PREFIX}$i
	i=i+1
done

# Action
[[ "$action" = "" ]] && action=status
case "$action" in
	"start")
		start $node
		;;
	"stop")
		stop $node
		;;
	"kill")
		kill $node
		;;
	"status")
		status $node
		;;
	*)
		usage
		;;
esac

echo
exit 0

